# Generated by h2py from /usr/include/sys/types.h

# Included from sys/isa_defs.h
_CHAR_ALIGNMENT = 1
_SHORT_ALIGNMENT = 2
_INT_ALIGNMENT = 4
_LONG_ALIGNMENT = 8
_LONG_LONG_ALIGNMENT = 8
_DOUBLE_ALIGNMENT = 8
_LONG_DOUBLE_ALIGNMENT = 16
_POINTER_ALIGNMENT = 8
_MAX_ALIGNMENT = 16
_ALIGNMENT_REQUIRED = 1
_CHAR_ALIGNMENT = 1
_SHORT_ALIGNMENT = 2
_INT_ALIGNMENT = 4
_LONG_ALIGNMENT = 4
_LONG_LONG_ALIGNMENT = 4
_DOUBLE_ALIGNMENT = 4
_LONG_DOUBLE_ALIGNMENT = 4
_POINTER_ALIGNMENT = 4
_MAX_ALIGNMENT = 4
_ALIGNMENT_REQUIRED = 0
_CHAR_ALIGNMENT = 1
_SHORT_ALIGNMENT = 2
_INT_ALIGNMENT = 4
_LONG_LONG_ALIGNMENT = 8
_DOUBLE_ALIGNMENT = 8
_ALIGNMENT_REQUIRED = 1
_LONG_ALIGNMENT = 4
_LONG_DOUBLE_ALIGNMENT = 8
_POINTER_ALIGNMENT = 4
_MAX_ALIGNMENT = 8
_LONG_ALIGNMENT = 8
_LONG_DOUBLE_ALIGNMENT = 16
_POINTER_ALIGNMENT = 8
_MAX_ALIGNMENT = 16

# Included from sys/feature_tests.h
_POSIX_C_SOURCE = 1
_LARGEFILE64_SOURCE = 1
_LARGEFILE_SOURCE = 1
_FILE_OFFSET_BITS = 64
_FILE_OFFSET_BITS = 32
_POSIX_C_SOURCE = 199506L
_POSIX_PTHREAD_SEMANTICS = 1
_XOPEN_VERSION = 500
_XOPEN_VERSION = 4
_XOPEN_VERSION = 3

# Included from sys/machtypes.h

# Included from sys/inttypes.h

# Included from sys/int_types.h

# Included from sys/int_limits.h
INT8_MAX = (127)
INT16_MAX = (32767)
INT32_MAX = (2147483647)
INTMAX_MAX = INT32_MAX
INT_LEAST8_MAX = INT8_MAX
INT_LEAST16_MAX = INT16_MAX
INT_LEAST32_MAX = INT32_MAX
INT8_MIN = (-128)
INT16_MIN = (-32767-1)
INT32_MIN = (-2147483647-1)
INTMAX_MIN = INT32_MIN
INT_LEAST8_MIN = INT8_MIN
INT_LEAST16_MIN = INT16_MIN
INT_LEAST32_MIN = INT32_MIN

# Included from sys/int_const.h
def INT8_C(c): return (c)

def INT16_C(c): return (c)

def INT32_C(c): return (c)

def INT64_C(c): return __CONCAT__(c,l)

def INT64_C(c): return __CONCAT__(c,ll)

def UINT8_C(c): return __CONCAT__(c,u)

def UINT16_C(c): return __CONCAT__(c,u)

def UINT32_C(c): return __CONCAT__(c,u)

def UINT64_C(c): return __CONCAT__(c,ul)

def UINT64_C(c): return __CONCAT__(c,ull)

def INTMAX_C(c): return __CONCAT__(c,l)

def UINTMAX_C(c): return __CONCAT__(c,ul)

def INTMAX_C(c): return __CONCAT__(c,ll)

def UINTMAX_C(c): return __CONCAT__(c,ull)

def INTMAX_C(c): return (c)

def UINTMAX_C(c): return (c)


# Included from sys/int_fmtio.h
PRId8 = "d"
PRId16 = "d"
PRId32 = "d"
PRId64 = "ld"
PRId64 = "lld"
PRIdLEAST8 = "d"
PRIdLEAST16 = "d"
PRIdLEAST32 = "d"
PRIdLEAST64 = "ld"
PRIdLEAST64 = "lld"
PRIi8 = "i"
PRIi16 = "i"
PRIi32 = "i"
PRIi64 = "li"
PRIi64 = "lli"
PRIiLEAST8 = "i"
PRIiLEAST16 = "i"
PRIiLEAST32 = "i"
PRIiLEAST64 = "li"
PRIiLEAST64 = "lli"
PRIo8 = "o"
PRIo16 = "o"
PRIo32 = "o"
PRIo64 = "lo"
PRIo64 = "llo"
PRIoLEAST8 = "o"
PRIoLEAST16 = "o"
PRIoLEAST32 = "o"
PRIoLEAST64 = "lo"
PRIoLEAST64 = "llo"
PRIx8 = "x"
PRIx16 = "x"
PRIx32 = "x"
PRIx64 = "lx"
PRIx64 = "llx"
PRIxLEAST8 = "x"
PRIxLEAST16 = "x"
PRIxLEAST32 = "x"
PRIxLEAST64 = "lx"
PRIxLEAST64 = "llx"
PRIX8 = "X"
PRIX16 = "X"
PRIX32 = "X"
PRIX64 = "lX"
PRIX64 = "llX"
PRIXLEAST8 = "X"
PRIXLEAST16 = "X"
PRIXLEAST32 = "X"
PRIXLEAST64 = "lX"
PRIXLEAST64 = "llX"
PRIu8 = "u"
PRIu16 = "u"
PRIu32 = "u"
PRIu64 = "lu"
PRIu64 = "llu"
PRIuLEAST8 = "u"
PRIuLEAST16 = "u"
PRIuLEAST32 = "u"
PRIuLEAST64 = "lu"
PRIuLEAST64 = "llu"
SCNd16 = "hd"
SCNd32 = "d"
SCNd64 = "ld"
SCNd64 = "lld"
SCNi16 = "hi"
SCNi32 = "i"
SCNi64 = "li"
SCNi64 = "lli"
SCNo16 = "ho"
SCNo32 = "o"
SCNo64 = "lo"
SCNo64 = "llo"
SCNu16 = "hu"
SCNu32 = "u"
SCNu64 = "lu"
SCNu64 = "llu"
SCNx16 = "hx"
SCNx32 = "x"
SCNx64 = "lx"
SCNx64 = "llx"
PRIdMAX = "ld"
PRIoMAX = "lo"
PRIxMAX = "lx"
PRIuMAX = "lu"
PRIdMAX = "lld"
PRIoMAX = "llo"
PRIxMAX = "llx"
PRIuMAX = "llu"
PRIdMAX = "d"
PRIoMAX = "o"
PRIxMAX = "x"
PRIuMAX = "u"
SCNiMAX = "li"
SCNdMAX = "ld"
SCNoMAX = "lo"
SCNxMAX = "lx"
SCNiMAX = "lli"
SCNdMAX = "lld"
SCNoMAX = "llo"
SCNxMAX = "llx"
SCNiMAX = "i"
SCNdMAX = "d"
SCNoMAX = "o"
SCNxMAX = "x"

# Included from sys/types32.h
SHRT_MIN = (-32768)
SHRT_MAX = 32767
USHRT_MAX = 65535
INT_MIN = (-2147483647-1)
INT_MAX = 2147483647
LONG_MIN = (-9223372036854775807L-1L)
LONG_MAX = 9223372036854775807L
LONG_MIN = (-2147483647L-1L)
LONG_MAX = 2147483647L
P_MYID = (-1)

# Included from sys/select.h

# Included from sys/time.h
TIME32_MAX = INT32_MAX
TIME32_MIN = INT32_MIN
def TIMEVAL_OVERFLOW(tv): return \

from TYPES import *
DST_NONE = 0
DST_USA = 1
DST_AUST = 2
DST_WET = 3
DST_MET = 4
DST_EET = 5
DST_CAN = 6
DST_GB = 7
DST_RUM = 8
DST_TUR = 9
DST_AUSTALT = 10
ITIMER_REAL = 0
ITIMER_VIRTUAL = 1
ITIMER_PROF = 2
ITIMER_REALPROF = 3
def ITIMERVAL_OVERFLOW(itv): return \

SEC = 1
MILLISEC = 1000
MICROSEC = 1000000
NANOSEC = 1000000000

# Included from sys/time_impl.h
def TIMESPEC_OVERFLOW(ts): return \

def ITIMERSPEC_OVERFLOW(it): return \

__CLOCK_REALTIME0 = 0
CLOCK_VIRTUAL = 1
CLOCK_PROF = 2
__CLOCK_REALTIME3 = 3
CLOCK_HIGHRES = 4
CLOCK_MAX = 5
CLOCK_REALTIME = __CLOCK_REALTIME3
CLOCK_REALTIME = __CLOCK_REALTIME0
TIMER_RELTIME = 0x0
TIMER_ABSTIME = 0x1

# Included from sys/mutex.h
from TYPES import *
def MUTEX_HELD(x): return (mutex_owned(x))

def TICK_TO_SEC(tick): return ((tick) / hz)

def SEC_TO_TICK(sec): return ((sec) * hz)

def TICK_TO_MSEC(tick): return \

def MSEC_TO_TICK(msec): return \

def MSEC_TO_TICK_ROUNDUP(msec): return \

def TICK_TO_USEC(tick): return ((tick) * usec_per_tick)

def USEC_TO_TICK(usec): return ((usec) / usec_per_tick)

def USEC_TO_TICK_ROUNDUP(usec): return \

def TICK_TO_NSEC(tick): return ((tick) * nsec_per_tick)

def NSEC_TO_TICK(nsec): return ((nsec) / nsec_per_tick)

def NSEC_TO_TICK_ROUNDUP(nsec): return \

def TIMEVAL_TO_TICK(tvp): return \

def TIMESTRUC_TO_TICK(tsp): return \


# Included from time.h
from TYPES import *

# Included from iso/time_iso.h
NULL = 0L
NULL = 0
CLOCKS_PER_SEC = 1000000
FD_SETSIZE = 65536
FD_SETSIZE = 1024
_NBBY = 8
NBBY = _NBBY
def FD_ZERO(p): return bzero((p), sizeof (*(p)))



"""TestCase for reseting File ID.
"""

import os
import shutil
import unittest

from test_all import db, test_support, get_new_environment_path, get_new_database_path

class FileidResetTestCase(unittest.TestCase):
    def setUp(self):
        self.db_path_1 = get_new_database_path()
        self.db_path_2 = get_new_database_path()
        self.db_env_path = get_new_environment_path()

    def test_fileid_reset(self):
        # create DB 1
        self.db1 = db.DB()
        self.db1.open(self.db_path_1, dbtype=db.DB_HASH, flags=(db.DB_CREATE|db.DB_EXCL))
        self.db1.put('spam', 'eggs')
        self.db1.close()

        shutil.copy(self.db_path_1, self.db_path_2)

        self.db2 = db.DB()
        self.db2.open(self.db_path_2, dbtype=db.DB_HASH)
        self.db2.put('spam', 'spam')
        self.db2.close()

        self.db_env = db.DBEnv()
        self.db_env.open(self.db_env_path, db.DB_CREATE|db.DB_INIT_MPOOL)

        # use fileid_reset() here
        self.db_env.fileid_reset(self.db_path_2)

        self.db1 = db.DB(self.db_env)
        self.db1.open(self.db_path_1, dbtype=db.DB_HASH, flags=db.DB_RDONLY)
        self.assertEqual(self.db1.get('spam'), 'eggs')

        self.db2 = db.DB(self.db_env)
        self.db2.open(self.db_path_2, dbtype=db.DB_HASH, flags=db.DB_RDONLY)
        self.assertEqual(self.db2.get('spam'), 'spam')

        self.db1.close()
        self.db2.close()

        self.db_env.close()

    def tearDown(self):
        test_support.unlink(self.db_path_1)
        test_support.unlink(self.db_path_2)
        test_support.rmtree(self.db_env_path)

def test_suite():
    suite = unittest.TestSuite()
    if db.version() >= (4, 4):
        suite.addTest(unittest.makeSuite(FileidResetTestCase))
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



import unittest
from test import test_support

from random import random
from math import atan2, isnan, copysign

INF = float("inf")
NAN = float("nan")
# These tests ensure that complex math does the right thing

# decorator for skipping tests on non-IEEE 754 platforms
have_getformat = hasattr(float, "__getformat__")
requires_IEEE_754 = unittest.skipUnless(have_getformat and
    float.__getformat__("double").startswith("IEEE"),
    "test requires IEEE 754 doubles")


class ComplexTest(unittest.TestCase):

    def assertAlmostEqual(self, a, b):
        if isinstance(a, complex):
            if isinstance(b, complex):
                unittest.TestCase.assertAlmostEqual(self, a.real, b.real)
                unittest.TestCase.assertAlmostEqual(self, a.imag, b.imag)
            else:
                unittest.TestCase.assertAlmostEqual(self, a.real, b)
                unittest.TestCase.assertAlmostEqual(self, a.imag, 0.)
        else:
            if isinstance(b, complex):
                unittest.TestCase.assertAlmostEqual(self, a, b.real)
                unittest.TestCase.assertAlmostEqual(self, 0., b.imag)
            else:
                unittest.TestCase.assertAlmostEqual(self, a, b)

    def assertCloseAbs(self, x, y, eps=1e-9):
        """Return true iff floats x and y "are close"."""
        # put the one with larger magnitude second
        if abs(x) > abs(y):
            x, y = y, x
        if y == 0:
            return abs(x) < eps
        if x == 0:
            return abs(y) < eps
        # check that relative difference < eps
        self.assertTrue(abs((x-y)/y) < eps)

    def assertFloatsAreIdentical(self, x, y):
        """assert that floats x and y are identical, in the sense that:
        (1) both x and y are nans, or
        (2) both x and y are infinities, with the same sign, or
        (3) both x and y are zeros, with the same sign, or
        (4) x and y are both finite and nonzero, and x == y

        """
        msg = 'floats {!r} and {!r} are not identical'

        if isnan(x) or isnan(y):
            if isnan(x) and isnan(y):
                return
        elif x == y:
            if x != 0.0:
                return
            # both zero; check that signs match
            elif copysign(1.0, x) == copysign(1.0, y):
                return
            else:
                msg += ': zeros have different signs'
        self.fail(msg.format(x, y))

    def assertClose(self, x, y, eps=1e-9):
        """Return true iff complexes x and y "are close"."""
        self.assertCloseAbs(x.real, y.real, eps)
        self.assertCloseAbs(x.imag, y.imag, eps)

    def check_div(self, x, y):
        """Compute complex z=x*y, and check that z/x==y and z/y==x."""
        z = x * y
        if x != 0:
            q = z / x
            self.assertClose(q, y)
            q = z.__div__(x)
            self.assertClose(q, y)
            q = z.__truediv__(x)
            self.assertClose(q, y)
        if y != 0:
            q = z / y
            self.assertClose(q, x)
            q = z.__div__(y)
            self.assertClose(q, x)
            q = z.__truediv__(y)
            self.assertClose(q, x)

    def test_div(self):
        simple_real = [float(i) for i in xrange(-5, 6)]
        simple_complex = [complex(x, y) for x in simple_real for y in simple_real]
        for x in simple_complex:
            for y in simple_complex:
                self.check_div(x, y)

        # A naive complex division algorithm (such as in 2.0) is very prone to
        # nonsense errors for these (overflows and underflows).
        self.check_div(complex(1e200, 1e200), 1+0j)
        self.check_div(complex(1e-200, 1e-200), 1+0j)

        # Just for fun.
        for i in xrange(100):
            self.check_div(complex(random(), random()),
                           complex(random(), random()))

        self.assertRaises(ZeroDivisionError, complex.__div__, 1+1j, 0+0j)
        # FIXME: The following currently crashes on Alpha
        # self.assertRaises(OverflowError, pow, 1e200+1j, 1e200+1j)

    def test_truediv(self):
        self.assertAlmostEqual(complex.__truediv__(2+0j, 1+1j), 1-1j)
        self.assertRaises(ZeroDivisionError, complex.__truediv__, 1+1j, 0+0j)

        for denom_real, denom_imag in [(0, NAN), (NAN, 0), (NAN, NAN)]:
            z = complex(0, 0) / complex(denom_real, denom_imag)
            self.assertTrue(isnan(z.real))
            self.assertTrue(isnan(z.imag))

    def test_floordiv(self):
        self.assertAlmostEqual(complex.__floordiv__(3+0j, 1.5+0j), 2)
        self.assertRaises(ZeroDivisionError, complex.__floordiv__, 3+0j, 0+0j)

    def test_coerce(self):
        self.assertRaises(OverflowError, complex.__coerce__, 1+1j, 1L<<10000)

    def test_no_implicit_coerce(self):
        # Python 2.7 removed implicit coercion from the complex type
        class A(object):
            def __coerce__(self, other):
                raise RuntimeError
            __hash__ = None
            def __cmp__(self, other):
                return -1

        a = A()
        self.assertRaises(TypeError, lambda: a + 2.0j)
        self.assertTrue(a < 2.0j)

    def test_richcompare(self):
        self.assertEqual(complex.__eq__(1+1j, 1L<<10000), False)
        self.assertEqual(complex.__lt__(1+1j, None), NotImplemented)
        self.assertIs(complex.__eq__(1+1j, 1+1j), True)
        self.assertIs(complex.__eq__(1+1j, 2+2j), False)
        self.assertIs(complex.__ne__(1+1j, 1+1j), False)
        self.assertIs(complex.__ne__(1+1j, 2+2j), True)
        self.assertRaises(TypeError, complex.__lt__, 1+1j, 2+2j)
        self.assertRaises(TypeError, complex.__le__, 1+1j, 2+2j)
        self.assertRaises(TypeError, complex.__gt__, 1+1j, 2+2j)
        self.assertRaises(TypeError, complex.__ge__, 1+1j, 2+2j)

    def test_richcompare_boundaries(self):
        def check(n, deltas, is_equal, imag = 0.0):
            for delta in deltas:
                i = n + delta
                z = complex(i, imag)
                self.assertIs(complex.__eq__(z, i), is_equal(delta))
                self.assertIs(complex.__ne__(z, i), not is_equal(delta))
        # For IEEE-754 doubles the following should hold:
        #    x in [2 ** (52 + i), 2 ** (53 + i + 1)] -> x mod 2 ** i == 0
        # where the interval is representable, of course.
        for i in range(1, 10):
            pow = 52 + i
            mult = 2 ** i
            check(2 ** pow, range(1, 101), lambda delta: delta % mult == 0)
            check(2 ** pow, range(1, 101), lambda delta: False, float(i))
        check(2 ** 53, range(-100, 0), lambda delta: True)

    def test_mod(self):
        self.assertRaises(ZeroDivisionError, (1+1j).__mod__, 0+0j)

        a = 3.33+4.43j
        try:
            a % 0
        except ZeroDivisionError:
            pass
        else:
            self.fail("modulo parama can't be 0")

    def test_divmod(self):
        self.assertRaises(ZeroDivisionError, divmod, 1+1j, 0+0j)

    def test_pow(self):
        self.assertAlmostEqual(pow(1+1j, 0+0j), 1.0)
        self.assertAlmostEqual(pow(0+0j, 2+0j), 0.0)
        self.assertRaises(ZeroDivisionError, pow, 0+0j, 1j)
        self.assertAlmostEqual(pow(1j, -1), 1/1j)
        self.assertAlmostEqual(pow(1j, 200), 1)
        self.assertRaises(ValueError, pow, 1+1j, 1+1j, 1+1j)

        a = 3.33+4.43j
        self.assertEqual(a ** 0j, 1)
        self.assertEqual(a ** 0.+0.j, 1)

        self.assertEqual(3j ** 0j, 1)
        self.assertEqual(3j ** 0, 1)

        try:
            0j ** a
        except ZeroDivisionError:
            pass
        else:
            self.fail("should fail 0.0 to negative or complex power")

        try:
            0j ** (3-2j)
        except ZeroDivisionError:
            pass
        else:
            self.fail("should fail 0.0 to negative or complex power")

        # The following is used to exercise certain code paths
        self.assertEqual(a ** 105, a ** 105)
        self.assertEqual(a ** -105, a ** -105)
        self.assertEqual(a ** -30, a ** -30)

        self.assertEqual(0.0j ** 0, 1)

        b = 5.1+2.3j
        self.assertRaises(ValueError, pow, a, b, 0)

    def test_boolcontext(self):
        for i in xrange(100):
            self.assertTrue(complex(random() + 1e-6, random() + 1e-6))
        self.assertTrue(not complex(0.0, 0.0))

    def test_conjugate(self):
        self.assertClose(complex(5.3, 9.8).conjugate(), 5.3-9.8j)

    def test_constructor(self):
        class OS:
            def __init__(self, value): self.value = value
            def __complex__(self): return self.value
        class NS(object):
            def __init__(self, value): self.value = value
            def __complex__(self): return self.value
        self.assertEqual(complex(OS(1+10j)), 1+10j)
        self.assertEqual(complex(NS(1+10j)), 1+10j)
        self.assertRaises(TypeError, complex, OS(None))
        self.assertRaises(TypeError, complex, NS(None))

        self.assertAlmostEqual(complex("1+10j"), 1+10j)
        self.assertAlmostEqual(complex(10), 10+0j)
        self.assertAlmostEqual(complex(10.0), 10+0j)
        self.assertAlmostEqual(complex(10L), 10+0j)
        self.assertAlmostEqual(complex(10+0j), 10+0j)
        self.assertAlmostEqual(complex(1,10), 1+10j)
        self.assertAlmostEqual(complex(1,10L), 1+10j)
        self.assertAlmostEqual(complex(1,10.0), 1+10j)
        self.assertAlmostEqual(complex(1L,10), 1+10j)
        self.assertAlmostEqual(complex(1L,10L), 1+10j)
        self.assertAlmostEqual(complex(1L,10.0), 1+10j)
        self.assertAlmostEqual(complex(1.0,10), 1+10j)
        self.assertAlmostEqual(complex(1.0,10L), 1+10j)
        self.assertAlmostEqual(complex(1.0,10.0), 1+10j)
        self.assertAlmostEqual(complex(3.14+0j), 3.14+0j)
        self.assertAlmostEqual(complex(3.14), 3.14+0j)
        self.assertAlmostEqual(complex(314), 314.0+0j)
        self.assertAlmostEqual(complex(314L), 314.0+0j)
        self.assertAlmostEqual(complex(3.14+0j, 0j), 3.14+0j)
        self.assertAlmostEqual(complex(3.14, 0.0), 3.14+0j)
        self.assertAlmostEqual(complex(314, 0), 314.0+0j)
        self.assertAlmostEqual(complex(314L, 0L), 314.0+0j)
        self.assertAlmostEqual(complex(0j, 3.14j), -3.14+0j)
        self.assertAlmostEqual(complex(0.0, 3.14j), -3.14+0j)
        self.assertAlmostEqual(complex(0j, 3.14), 3.14j)
        self.assertAlmostEqual(complex(0.0, 3.14), 3.14j)
        self.assertAlmostEqual(complex("1"), 1+0j)
        self.assertAlmostEqual(complex("1j"), 1j)
        self.assertAlmostEqual(complex(),  0)
        self.assertAlmostEqual(complex("-1"), -1)
        self.assertAlmostEqual(complex("+1"), +1)
        self.assertAlmostEqual(complex("(1+2j)"), 1+2j)
        self.assertAlmostEqual(complex("(1.3+2.2j)"), 1.3+2.2j)
        self.assertAlmostEqual(complex("3.14+1J"), 3.14+1j)
        self.assertAlmostEqual(complex(" ( +3.14-6J )"), 3.14-6j)
        self.assertAlmostEqual(complex(" ( +3.14-J )"), 3.14-1j)
        self.assertAlmostEqual(complex(" ( +3.14+j )"), 3.14+1j)
        self.assertAlmostEqual(complex("J"), 1j)
        self.assertAlmostEqual(complex("( j )"), 1j)
        self.assertAlmostEqual(complex("+J"), 1j)
        self.assertAlmostEqual(complex("( -j)"), -1j)
        self.assertAlmostEqual(complex('1e-500'), 0.0 + 0.0j)
        self.assertAlmostEqual(complex('-1e-500j'), 0.0 - 0.0j)
        self.assertAlmostEqual(complex('-1e-500+1e-500j'), -0.0 + 0.0j)

        class complex2(complex): pass
        self.assertAlmostEqual(complex(complex2(1+1j)), 1+1j)
        self.assertAlmostEqual(complex(real=17, imag=23), 17+23j)
        self.assertAlmostEqual(complex(real=17+23j), 17+23j)
        self.assertAlmostEqual(complex(real=17+23j, imag=23), 17+46j)
        self.assertAlmostEqual(complex(real=1+2j, imag=3+4j), -3+5j)

        # check that the sign of a zero in the real or imaginary part
        # is preserved when constructing from two floats.  (These checks
        # are harmless on systems without support for signed zeros.)
        def split_zeros(x):
            """Function that produces different results for 0. and -0."""
            return atan2(x, -1.)

        self.assertEqual(split_zeros(complex(1., 0.).imag), split_zeros(0.))
        self.assertEqual(split_zeros(complex(1., -0.).imag), split_zeros(-0.))
        self.assertEqual(split_zeros(complex(0., 1.).real), split_zeros(0.))
        self.assertEqual(split_zeros(complex(-0., 1.).real), split_zeros(-0.))

        c = 3.14 + 1j
        self.assertTrue(complex(c) is c)
        del c

        self.assertRaises(TypeError, complex, "1", "1")
        self.assertRaises(TypeError, complex, 1, "1")

        if test_support.have_unicode:
            self.assertEqual(complex(unicode("  3.14+J  ")), 3.14+1j)

        # SF bug 543840:  complex(string) accepts strings with \0
        # Fixed in 2.3.
        self.assertRaises(ValueError, complex, '1+1j\0j')

        self.assertRaises(TypeError, int, 5+3j)
        self.assertRaises(TypeError, long, 5+3j)
        self.assertRaises(TypeError, float, 5+3j)
        self.assertRaises(ValueError, complex, "")
        self.assertRaises(TypeError, complex, None)
        self.assertRaises(ValueError, complex, "\0")
        self.assertRaises(ValueError, complex, "3\09")
        self.assertRaises(TypeError, complex, "1", "2")
        self.assertRaises(TypeError, complex, "1", 42)
        self.assertRaises(TypeError, complex, 1, "2")
        self.assertRaises(ValueError, complex, "1+")
        self.assertRaises(ValueError, complex, "1+1j+1j")
        self.assertRaises(ValueError, complex, "--")
        self.assertRaises(ValueError, complex, "(1+2j")
        self.assertRaises(ValueError, complex, "1+2j)")
        self.assertRaises(ValueError, complex, "1+(2j)")
        self.assertRaises(ValueError, complex, "(1+2j)123")
        if test_support.have_unicode:
            self.assertRaises(ValueError, complex, unicode("x"))
        self.assertRaises(ValueError, complex, "1j+2")
        self.assertRaises(ValueError, complex, "1e1ej")
        self.assertRaises(ValueError, complex, "1e++1ej")
        self.assertRaises(ValueError, complex, ")1+2j(")
        # the following three are accepted by Python 2.6
        self.assertRaises(ValueError, complex, "1..1j")
        self.assertRaises(ValueError, complex, "1.11.1j")
        self.assertRaises(ValueError, complex, "1e1.1j")

        if test_support.have_unicode:
            # check that complex accepts long unicode strings
            self.assertEqual(type(complex(unicode("1"*500))), complex)

        class EvilExc(Exception):
            pass

        class evilcomplex:
            def __complex__(self):
                raise EvilExc

        self.assertRaises(EvilExc, complex, evilcomplex())

        class float2:
            def __init__(self, value):
                self.value = value
            def __float__(self):
                return self.value

        self.assertAlmostEqual(complex(float2(42.)), 42)
        self.assertAlmostEqual(complex(real=float2(17.), imag=float2(23.)), 17+23j)
        self.assertRaises(TypeError, complex, float2(None))

        class complex0(complex):
            """Test usage of __complex__() when inheriting from 'complex'"""
            def __complex__(self):
                return 42j

        class complex1(complex):
            """Test usage of __complex__() with a __new__() method"""
            def __new__(self, value=0j):
                return complex.__new__(self, 2*value)
            def __complex__(self):
                return self

        class complex2(complex):
            """Make sure that __complex__() calls fail if anything other than a
            complex is returned"""
            def __complex__(self):
                return None

        self.assertAlmostEqual(complex(complex0(1j)), 42j)
        self.assertAlmostEqual(complex(complex1(1j)), 2j)
        self.assertRaises(TypeError, complex, complex2(1j))

    def test_subclass(self):
        class xcomplex(complex):
            def __add__(self,other):
                return xcomplex(complex(self) + other)
            __radd__ = __add__

            def __sub__(self,other):
                return xcomplex(complex(self) + other)
            __rsub__ = __sub__

            def __mul__(self,other):
                return xcomplex(complex(self) * other)
            __rmul__ = __mul__

            def __div__(self,other):
                return xcomplex(complex(self) / other)

            def __rdiv__(self,other):
                return xcomplex(other / complex(self))

            __truediv__ = __div__
            __rtruediv__ = __rdiv__

            def __floordiv__(self,other):
                return xcomplex(complex(self) // other)

            def __rfloordiv__(self,other):
                return xcomplex(other // complex(self))

            def __pow__(self,other):
                return xcomplex(complex(self) ** other)

            def __rpow__(self,other):
                return xcomplex(other ** complex(self) )

            def __mod__(self,other):
                return xcomplex(complex(self) % other)

            def __rmod__(self,other):
                return xcomplex(other % complex(self))

        infix_binops = ('+', '-', '*', '**', '%', '//', '/')
        xcomplex_values = (xcomplex(1), xcomplex(123.0),
                           xcomplex(-10+2j), xcomplex(3+187j),
                           xcomplex(3-78j))
        test_values = (1, 123.0, 10-19j, xcomplex(1+2j),
                       xcomplex(1+87j), xcomplex(10+90j))

        for op in infix_binops:
            for x in xcomplex_values:
                for y in test_values:
                    a = 'x %s y' % op
                    b = 'y %s x' % op
                    self.assertTrue(type(eval(a)) is type(eval(b)) is xcomplex)

    @requires_IEEE_754
    def test_constructor_special_numbers(self):
        class complex2(complex):
            pass
        for x in 0.0, -0.0, INF, -INF, NAN:
            for y in 0.0, -0.0, INF, -INF, NAN:
                z = complex(x, y)
                self.assertFloatsAreIdentical(z.real, x)
                self.assertFloatsAreIdentical(z.imag, y)
                z = complex2(x, y)
                self.assertIs(type(z), complex2)
                self.assertFloatsAreIdentical(z.real, x)
                self.assertFloatsAreIdentical(z.imag, y)
                z = complex(complex2(x, y))
                self.assertIs(type(z), complex)
                self.assertFloatsAreIdentical(z.real, x)
                self.assertFloatsAreIdentical(z.imag, y)
                z = complex2(complex(x, y))
                self.assertIs(type(z), complex2)
                self.assertFloatsAreIdentical(z.real, x)
                self.assertFloatsAreIdentical(z.imag, y)

    def test_hash(self):
        for x in xrange(-30, 30):
            self.assertEqual(hash(x), hash(complex(x, 0)))
            x /= 3.0    # now check against floating point
            self.assertEqual(hash(x), hash(complex(x, 0.)))

    def test_abs(self):
        nums = [complex(x/3., y/7.) for x in xrange(-9,9) for y in xrange(-9,9)]
        for num in nums:
            self.assertAlmostEqual((num.real**2 + num.imag**2)  ** 0.5, abs(num))

    def test_repr(self):
        self.assertEqual(repr(1+6j), '(1+6j)')
        self.assertEqual(repr(1-6j), '(1-6j)')

        self.assertNotEqual(repr(-(1+0j)), '(-1+-0j)')

        self.assertEqual(1-6j,complex(repr(1-6j)))
        self.assertEqual(1+6j,complex(repr(1+6j)))
        self.assertEqual(-6j,complex(repr(-6j)))
        self.assertEqual(6j,complex(repr(6j)))

        self.assertEqual(repr(complex(1., INF)), "(1+infj)")
        self.assertEqual(repr(complex(1., -INF)), "(1-infj)")
        self.assertEqual(repr(complex(INF, 1)), "(inf+1j)")
        self.assertEqual(repr(complex(-INF, INF)), "(-inf+infj)")
        self.assertEqual(repr(complex(NAN, 1)), "(nan+1j)")
        self.assertEqual(repr(complex(1, NAN)), "(1+nanj)")
        self.assertEqual(repr(complex(NAN, NAN)), "(nan+nanj)")

        self.assertEqual(repr(complex(0, INF)), "infj")
        self.assertEqual(repr(complex(0, -INF)), "-infj")
        self.assertEqual(repr(complex(0, NAN)), "nanj")

    def test_neg(self):
        self.assertEqual(-(1+6j), -1-6j)

    def test_file(self):
        a = 3.33+4.43j
        b = 5.1+2.3j

        fo = None
        try:
            fo = open(test_support.TESTFN, "wb")
            print >>fo, a, b
            fo.close()
            fo = open(test_support.TESTFN, "rb")
            self.assertEqual(fo.read(), "%s %s\n" % (a, b))
        finally:
            if (fo is not None) and (not fo.closed):
                fo.close()
            test_support.unlink(test_support.TESTFN)

    def test_getnewargs(self):
        self.assertEqual((1+2j).__getnewargs__(), (1.0, 2.0))
        self.assertEqual((1-2j).__getnewargs__(), (1.0, -2.0))
        self.assertEqual((2j).__getnewargs__(), (0.0, 2.0))
        self.assertEqual((-0j).__getnewargs__(), (0.0, -0.0))
        self.assertEqual(complex(0, INF).__getnewargs__(), (0.0, INF))
        self.assertEqual(complex(INF, 0).__getnewargs__(), (INF, 0.0))

    if float.__getformat__("double").startswith("IEEE"):
        def test_plus_minus_0j(self):
            # test that -0j and 0j literals are not identified
            z1, z2 = 0j, -0j
            self.assertEqual(atan2(z1.imag, -1.), atan2(0., -1.))
            self.assertEqual(atan2(z2.imag, -1.), atan2(-0., -1.))

    @unittest.skipUnless(float.__getformat__("double").startswith("IEEE"),
                         "test requires IEEE 754 doubles")
    def test_overflow(self):
        self.assertEqual(complex("1e500"), complex(INF, 0.0))
        self.assertEqual(complex("-1e500j"), complex(0.0, -INF))
        self.assertEqual(complex("-1e500+1.8e308j"), complex(-INF, INF))

    @unittest.skipUnless(float.__getformat__("double").startswith("IEEE"),
                         "test requires IEEE 754 doubles")
    def test_repr_roundtrip(self):
        vals = [0.0, 1e-500, 1e-315, 1e-200, 0.0123, 3.1415, 1e50, INF, NAN]
        vals += [-v for v in vals]

        # complex(repr(z)) should recover z exactly, even for complex
        # numbers involving an infinity, nan, or negative zero
        for x in vals:
            for y in vals:
                z = complex(x, y)
                roundtrip = complex(repr(z))
                self.assertFloatsAreIdentical(z.real, roundtrip.real)
                self.assertFloatsAreIdentical(z.imag, roundtrip.imag)

        # if we predefine some constants, then eval(repr(z)) should
        # also work, except that it might change the sign of zeros
        inf, nan = float('inf'), float('nan')
        infj, nanj = complex(0.0, inf), complex(0.0, nan)
        for x in vals:
            for y in vals:
                z = complex(x, y)
                roundtrip = eval(repr(z))
                # adding 0.0 has no effect beside changing -0.0 to 0.0
                self.assertFloatsAreIdentical(0.0 + z.real,
                                              0.0 + roundtrip.real)
                self.assertFloatsAreIdentical(0.0 + z.imag,
                                              0.0 + roundtrip.imag)

    def test_format(self):
        # empty format string is same as str()
        self.assertEqual(format(1+3j, ''), str(1+3j))
        self.assertEqual(format(1.5+3.5j, ''), str(1.5+3.5j))
        self.assertEqual(format(3j, ''), str(3j))
        self.assertEqual(format(3.2j, ''), str(3.2j))
        self.assertEqual(format(3+0j, ''), str(3+0j))
        self.assertEqual(format(3.2+0j, ''), str(3.2+0j))

        # empty presentation type should still be analogous to str,
        # even when format string is nonempty (issue #5920).
        self.assertEqual(format(3.2+0j, '-'), str(3.2+0j))
        self.assertEqual(format(3.2+0j, '<'), str(3.2+0j))
        z = 4/7. - 100j/7.
        self.assertEqual(format(z, ''), str(z))
        self.assertEqual(format(z, '-'), str(z))
        self.assertEqual(format(z, '<'), str(z))
        self.assertEqual(format(z, '10'), str(z))
        z = complex(0.0, 3.0)
        self.assertEqual(format(z, ''), str(z))
        self.assertEqual(format(z, '-'), str(z))
        self.assertEqual(format(z, '<'), str(z))
        self.assertEqual(format(z, '2'), str(z))
        z = complex(-0.0, 2.0)
        self.assertEqual(format(z, ''), str(z))
        self.assertEqual(format(z, '-'), str(z))
        self.assertEqual(format(z, '<'), str(z))
        self.assertEqual(format(z, '3'), str(z))

        self.assertEqual(format(1+3j, 'g'), '1+3j')
        self.assertEqual(format(3j, 'g'), '0+3j')
        self.assertEqual(format(1.5+3.5j, 'g'), '1.5+3.5j')

        self.assertEqual(format(1.5+3.5j, '+g'), '+1.5+3.5j')
        self.assertEqual(format(1.5-3.5j, '+g'), '+1.5-3.5j')
        self.assertEqual(format(1.5-3.5j, '-g'), '1.5-3.5j')
        self.assertEqual(format(1.5+3.5j, ' g'), ' 1.5+3.5j')
        self.assertEqual(format(1.5-3.5j, ' g'), ' 1.5-3.5j')
        self.assertEqual(format(-1.5+3.5j, ' g'), '-1.5+3.5j')
        self.assertEqual(format(-1.5-3.5j, ' g'), '-1.5-3.5j')

        self.assertEqual(format(-1.5-3.5e-20j, 'g'), '-1.5-3.5e-20j')
        self.assertEqual(format(-1.5-3.5j, 'f'), '-1.500000-3.500000j')
        self.assertEqual(format(-1.5-3.5j, 'F'), '-1.500000-3.500000j')
        self.assertEqual(format(-1.5-3.5j, 'e'), '-1.500000e+00-3.500000e+00j')
        self.assertEqual(format(-1.5-3.5j, '.2e'), '-1.50e+00-3.50e+00j')
        self.assertEqual(format(-1.5-3.5j, '.2E'), '-1.50E+00-3.50E+00j')
        self.assertEqual(format(-1.5e10-3.5e5j, '.2G'), '-1.5E+10-3.5E+05j')

        self.assertEqual(format(1.5+3j, '<20g'),  '1.5+3j              ')
        self.assertEqual(format(1.5+3j, '*<20g'), '1.5+3j**************')
        self.assertEqual(format(1.5+3j, '>20g'),  '              1.5+3j')
        self.assertEqual(format(1.5+3j, '^20g'),  '       1.5+3j       ')
        self.assertEqual(format(1.5+3j, '<20'),   '(1.5+3j)            ')
        self.assertEqual(format(1.5+3j, '>20'),   '            (1.5+3j)')
        self.assertEqual(format(1.5+3j, '^20'),   '      (1.5+3j)      ')
        self.assertEqual(format(1.123-3.123j, '^20.2'), '     (1.1-3.1j)     ')

        self.assertEqual(format(1.5+3j, '20.2f'), '          1.50+3.00j')
        self.assertEqual(format(1.5+3j, '>20.2f'), '          1.50+3.00j')
        self.assertEqual(format(1.5+3j, '<20.2f'), '1.50+3.00j          ')
        self.assertEqual(format(1.5e20+3j, '<20.2f'), '150000000000000000000.00+3.00j')
        self.assertEqual(format(1.5e20+3j, '>40.2f'), '          150000000000000000000.00+3.00j')
        self.assertEqual(format(1.5e20+3j, '^40,.2f'), '  150,000,000,000,000,000,000.00+3.00j  ')
        self.assertEqual(format(1.5e21+3j, '^40,.2f'), ' 1,500,000,000,000,000,000,000.00+3.00j ')
        self.assertEqual(format(1.5e21+3000j, ',.2f'), '1,500,000,000,000,000,000,000.00+3,000.00j')

        # alternate is invalid
        self.assertRaises(ValueError, (1.5+0.5j).__format__, '#f')

        # zero padding is invalid
        self.assertRaises(ValueError, (1.5+0.5j).__format__, '010f')

        # '=' alignment is invalid
        self.assertRaises(ValueError, (1.5+3j).__format__, '=20')

        # integer presentation types are an error
        for t in 'bcdoxX':
            self.assertRaises(ValueError, (1.5+0.5j).__format__, t)

        # make sure everything works in ''.format()
        self.assertEqual('*{0:.3f}*'.format(3.14159+2.71828j), '*3.142+2.718j*')

        # issue 3382: 'f' and 'F' with inf's and nan's
        self.assertEqual('{0:f}'.format(INF+0j), 'inf+0.000000j')
        self.assertEqual('{0:F}'.format(INF+0j), 'INF+0.000000j')
        self.assertEqual('{0:f}'.format(-INF+0j), '-inf+0.000000j')
        self.assertEqual('{0:F}'.format(-INF+0j), '-INF+0.000000j')
        self.assertEqual('{0:f}'.format(complex(INF, INF)), 'inf+infj')
        self.assertEqual('{0:F}'.format(complex(INF, INF)), 'INF+INFj')
        self.assertEqual('{0:f}'.format(complex(INF, -INF)), 'inf-infj')
        self.assertEqual('{0:F}'.format(complex(INF, -INF)), 'INF-INFj')
        self.assertEqual('{0:f}'.format(complex(-INF, INF)), '-inf+infj')
        self.assertEqual('{0:F}'.format(complex(-INF, INF)), '-INF+INFj')
        self.assertEqual('{0:f}'.format(complex(-INF, -INF)), '-inf-infj')
        self.assertEqual('{0:F}'.format(complex(-INF, -INF)), '-INF-INFj')

        self.assertEqual('{0:f}'.format(complex(NAN, 0)), 'nan+0.000000j')
        self.assertEqual('{0:F}'.format(complex(NAN, 0)), 'NAN+0.000000j')
        self.assertEqual('{0:f}'.format(complex(NAN, NAN)), 'nan+nanj')
        self.assertEqual('{0:F}'.format(complex(NAN, NAN)), 'NAN+NANj')

def test_main():
    with test_support.check_warnings(("complex divmod.., // and % are "
                                      "deprecated", DeprecationWarning)):
        test_support.run_unittest(ComplexTest)

if __name__ == "__main__":
    test_main()



#
# Analogue of `multiprocessing.connection` which uses queues instead of sockets
#
# multiprocessing/dummy/connection.py
#
# Copyright (c) 2006-2008, R Oudkerk
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of author nor the names of any contributors may be
#    used to endorse or promote products derived from this software
#    without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#

__all__ = [ 'Client', 'Listener', 'Pipe' ]

from Queue import Queue


families = [None]


class Listener(object):

    def __init__(self, address=None, family=None, backlog=1):
        self._backlog_queue = Queue(backlog)

    def accept(self):
        return Connection(*self._backlog_queue.get())

    def close(self):
        self._backlog_queue = None

    address = property(lambda self: self._backlog_queue)


def Client(address):
    _in, _out = Queue(), Queue()
    address.put((_out, _in))
    return Connection(_in, _out)


def Pipe(duplex=True):
    a, b = Queue(), Queue()
    return Connection(a, b), Connection(b, a)


class Connection(object):

    def __init__(self, _in, _out):
        self._out = _out
        self._in = _in
        self.send = self.send_bytes = _out.put
        self.recv = self.recv_bytes = _in.get

    def poll(self, timeout=0.0):
        if self._in.qsize() > 0:
            return True
        if timeout <= 0.0:
            return False
        self._in.not_empty.acquire()
        self._in.not_empty.wait(timeout)
        self._in.not_empty.release()
        return self._in.qsize() > 0

    def close(self):
        pass



"""
gensuitemodule - Generate an AE suite module from an aete/aeut resource

Based on aete.py.

Reading and understanding this code is left as an exercise to the reader.
"""

from warnings import warnpy3k
warnpy3k("In 3.x, the gensuitemodule module is removed.", stacklevel=2)

import MacOS
import EasyDialogs
import os
import string
import sys
import types
import StringIO
import keyword
import macresource
import aetools
import distutils.sysconfig
import OSATerminology
from Carbon.Res import *
import Carbon.Folder
import MacOS
import getopt
import plistlib

_MAC_LIB_FOLDER=os.path.dirname(aetools.__file__)
DEFAULT_STANDARD_PACKAGEFOLDER=os.path.join(_MAC_LIB_FOLDER, 'lib-scriptpackages')
DEFAULT_USER_PACKAGEFOLDER=distutils.sysconfig.get_python_lib()

def usage():
    sys.stderr.write("Usage: %s [opts] application-or-resource-file\n" % sys.argv[0])
    sys.stderr.write("""Options:
--output pkgdir  Pathname of the output package (short: -o)
--resource       Parse resource file in stead of launching application (-r)
--base package   Use another base package in stead of default StdSuites (-b)
--edit old=new   Edit suite names, use empty new to skip a suite (-e)
--creator code   Set creator code for package (-c)
--dump           Dump aete resource to stdout in stead of creating module (-d)
--verbose        Tell us what happens (-v)
""")
    sys.exit(1)

def main():
    if len(sys.argv) > 1:
        SHORTOPTS = "rb:o:e:c:dv"
        LONGOPTS = ("resource", "base=", "output=", "edit=", "creator=", "dump", "verbose")
        try:
            opts, args = getopt.getopt(sys.argv[1:], SHORTOPTS, LONGOPTS)
        except getopt.GetoptError:
            usage()

        process_func = processfile
        basepkgname = 'StdSuites'
        output = None
        edit_modnames = []
        creatorsignature = None
        dump = None
        verbose = None

        for o, a in opts:
            if o in ('-r', '--resource'):
                process_func = processfile_fromresource
            if o in ('-b', '--base'):
                basepkgname = a
            if o in ('-o', '--output'):
                output = a
            if o in ('-e', '--edit'):
                split = a.split('=')
                if len(split) != 2:
                    usage()
                edit_modnames.append(split)
            if o in ('-c', '--creator'):
                if len(a) != 4:
                    sys.stderr.write("creator must be 4-char string\n")
                    sys.exit(1)
                creatorsignature = a
            if o in ('-d', '--dump'):
                dump = sys.stdout
            if o in ('-v', '--verbose'):
                verbose = sys.stderr


        if output and len(args) > 1:
            sys.stderr.write("%s: cannot specify --output with multiple inputs\n" % sys.argv[0])
            sys.exit(1)

        for filename in args:
            process_func(filename, output=output, basepkgname=basepkgname,
                edit_modnames=edit_modnames, creatorsignature=creatorsignature,
                dump=dump, verbose=verbose)
    else:
        main_interactive()

def main_interactive(interact=0, basepkgname='StdSuites'):
    if interact:
        # Ask for save-filename for each module
        edit_modnames = None
    else:
        # Use default filenames for each module
        edit_modnames = []
    appsfolder = Carbon.Folder.FSFindFolder(-32765, 'apps', 0)
    filename = EasyDialogs.AskFileForOpen(
        message='Select scriptable application',
        dialogOptionFlags=0x1056,       # allow selection of .app bundles
        defaultLocation=appsfolder)
    if not filename:
        return
    if not is_scriptable(filename):
        if EasyDialogs.AskYesNoCancel(
                "Warning: application does not seem scriptable",
                yes="Continue", default=2, no="") <= 0:
            return
    try:
        processfile(filename, edit_modnames=edit_modnames, basepkgname=basepkgname,
        verbose=sys.stderr)
    except MacOS.Error, arg:
        print "Error getting terminology:", arg
        print "Retry, manually parsing resources"
        processfile_fromresource(filename, edit_modnames=edit_modnames,
            basepkgname=basepkgname, verbose=sys.stderr)

def is_scriptable(application):
    """Return true if the application is scriptable"""
    if os.path.isdir(application):
        plistfile = os.path.join(application, 'Contents', 'Info.plist')
        if not os.path.exists(plistfile):
            return False
        plist = plistlib.Plist.fromFile(plistfile)
        return plist.get('NSAppleScriptEnabled', False)
    # If it is a file test for an aete/aeut resource.
    currf = CurResFile()
    try:
        refno = macresource.open_pathname(application)
    except MacOS.Error:
        return False
    UseResFile(refno)
    n_terminology = Count1Resources('aete') + Count1Resources('aeut') + \
        Count1Resources('scsz') + Count1Resources('osiz')
    CloseResFile(refno)
    UseResFile(currf)
    return n_terminology > 0

def processfile_fromresource(fullname, output=None, basepkgname=None,
        edit_modnames=None, creatorsignature=None, dump=None, verbose=None):
    """Process all resources in a single file"""
    if not is_scriptable(fullname) and verbose:
        print >>verbose, "Warning: app does not seem scriptable: %s" % fullname
    cur = CurResFile()
    if verbose:
        print >>verbose, "Processing", fullname
    rf = macresource.open_pathname(fullname)
    try:
        UseResFile(rf)
        resources = []
        for i in range(Count1Resources('aete')):
            res = Get1IndResource('aete', 1+i)
            resources.append(res)
        for i in range(Count1Resources('aeut')):
            res = Get1IndResource('aeut', 1+i)
            resources.append(res)
        if verbose:
            print >>verbose, "\nLISTING aete+aeut RESOURCES IN", repr(fullname)
        aetelist = []
        for res in resources:
            if verbose:
                print >>verbose, "decoding", res.GetResInfo(), "..."
            data = res.data
            aete = decode(data, verbose)
            aetelist.append((aete, res.GetResInfo()))
    finally:
        if rf != cur:
            CloseResFile(rf)
            UseResFile(cur)
    # switch back (needed for dialogs in Python)
    UseResFile(cur)
    if dump:
        dumpaetelist(aetelist, dump)
    compileaetelist(aetelist, fullname, output=output,
        basepkgname=basepkgname, edit_modnames=edit_modnames,
        creatorsignature=creatorsignature, verbose=verbose)

def processfile(fullname, output=None, basepkgname=None,
        edit_modnames=None, creatorsignature=None, dump=None,
        verbose=None):
    """Ask an application for its terminology and process that"""
    if not is_scriptable(fullname) and verbose:
        print >>verbose, "Warning: app does not seem scriptable: %s" % fullname
    if verbose:
        print >>verbose, "\nASKING FOR aete DICTIONARY IN", repr(fullname)
    try:
        aedescobj, launched = OSATerminology.GetAppTerminology(fullname)
    except MacOS.Error, arg:
        if arg[0] in (-1701, -192): # errAEDescNotFound, resNotFound
            if verbose:
                print >>verbose, "GetAppTerminology failed with errAEDescNotFound/resNotFound, trying manually"
            aedata, sig = getappterminology(fullname, verbose=verbose)
            if not creatorsignature:
                creatorsignature = sig
        else:
            raise
    else:
        if launched:
            if verbose:
                print >>verbose, "Launched", fullname
        raw = aetools.unpack(aedescobj)
        if not raw:
            if verbose:
                print >>verbose, 'Unpack returned empty value:', raw
            return
        if not raw[0].data:
            if verbose:
                print >>verbose, 'Unpack returned value without data:', raw
            return
        aedata = raw[0]
    aete = decode(aedata.data, verbose)
    if dump:
        dumpaetelist([aete], dump)
        return
    compileaete(aete, None, fullname, output=output, basepkgname=basepkgname,
        creatorsignature=creatorsignature, edit_modnames=edit_modnames,
        verbose=verbose)

def getappterminology(fullname, verbose=None):
    """Get application terminology by sending an AppleEvent"""
    # First check that we actually can send AppleEvents
    if not MacOS.WMAvailable():
        raise RuntimeError, "Cannot send AppleEvents, no access to window manager"
    # Next, a workaround for a bug in MacOS 10.2: sending events will hang unless
    # you have created an event loop first.
    import Carbon.Evt
    Carbon.Evt.WaitNextEvent(0,0)
    if os.path.isdir(fullname):
        # Now get the signature of the application, hoping it is a bundle
        pkginfo = os.path.join(fullname, 'Contents', 'PkgInfo')
        if not os.path.exists(pkginfo):
            raise RuntimeError, "No PkgInfo file found"
        tp_cr = open(pkginfo, 'rb').read()
        cr = tp_cr[4:8]
    else:
        # Assume it is a file
        cr, tp = MacOS.GetCreatorAndType(fullname)
    # Let's talk to it and ask for its AETE
    talker = aetools.TalkTo(cr)
    try:
        talker._start()
    except (MacOS.Error, aetools.Error), arg:
        if verbose:
            print >>verbose, 'Warning: start() failed, continuing anyway:', arg
    reply = talker.send("ascr", "gdte")
    #reply2 = talker.send("ascr", "gdut")
    # Now pick the bits out of the return that we need.
    return reply[1]['----'], cr


def compileaetelist(aetelist, fullname, output=None, basepkgname=None,
            edit_modnames=None, creatorsignature=None, verbose=None):
    for aete, resinfo in aetelist:
        compileaete(aete, resinfo, fullname, output=output,
            basepkgname=basepkgname, edit_modnames=edit_modnames,
            creatorsignature=creatorsignature, verbose=verbose)

def dumpaetelist(aetelist, output):
    import pprint
    pprint.pprint(aetelist, output)

def decode(data, verbose=None):
    """Decode a resource into a python data structure"""
    f = StringIO.StringIO(data)
    aete = generic(getaete, f)
    aete = simplify(aete)
    processed = f.tell()
    unprocessed = len(f.read())
    total = f.tell()
    if unprocessed and verbose:
        verbose.write("%d processed + %d unprocessed = %d total\n" %
                         (processed, unprocessed, total))
    return aete

def simplify(item):
    """Recursively replace singleton tuples by their constituent item"""
    if type(item) is types.ListType:
        return map(simplify, item)
    elif type(item) == types.TupleType and len(item) == 2:
        return simplify(item[1])
    else:
        return item


# Here follows the aete resource decoder.
# It is presented bottom-up instead of top-down because there are  direct
# references to the lower-level part-decoders from the high-level part-decoders.

def getbyte(f, *args):
    c = f.read(1)
    if not c:
        raise EOFError, 'in getbyte' + str(args)
    return ord(c)

def getword(f, *args):
    getalign(f)
    s = f.read(2)
    if len(s) < 2:
        raise EOFError, 'in getword' + str(args)
    return (ord(s[0])<<8) | ord(s[1])

def getlong(f, *args):
    getalign(f)
    s = f.read(4)
    if len(s) < 4:
        raise EOFError, 'in getlong' + str(args)
    return (ord(s[0])<<24) | (ord(s[1])<<16) | (ord(s[2])<<8) | ord(s[3])

def getostype(f, *args):
    getalign(f)
    s = f.read(4)
    if len(s) < 4:
        raise EOFError, 'in getostype' + str(args)
    return s

def getpstr(f, *args):
    c = f.read(1)
    if len(c) < 1:
        raise EOFError, 'in getpstr[1]' + str(args)
    nbytes = ord(c)
    if nbytes == 0: return ''
    s = f.read(nbytes)
    if len(s) < nbytes:
        raise EOFError, 'in getpstr[2]' + str(args)
    return s

def getalign(f):
    if f.tell() & 1:
        c = f.read(1)
        ##if c != '\0':
        ##  print align:', repr(c)

def getlist(f, description, getitem):
    count = getword(f)
    list = []
    for i in range(count):
        list.append(generic(getitem, f))
        getalign(f)
    return list

def alt_generic(what, f, *args):
    print "generic", repr(what), args
    res = vageneric(what, f, args)
    print '->', repr(res)
    return res

def generic(what, f, *args):
    if type(what) == types.FunctionType:
        return apply(what, (f,) + args)
    if type(what) == types.ListType:
        record = []
        for thing in what:
            item = apply(generic, thing[:1] + (f,) + thing[1:])
            record.append((thing[1], item))
        return record
    return "BAD GENERIC ARGS: %r" % (what,)

getdata = [
    (getostype, "type"),
    (getpstr, "description"),
    (getword, "flags")
    ]
getargument = [
    (getpstr, "name"),
    (getostype, "keyword"),
    (getdata, "what")
    ]
getevent = [
    (getpstr, "name"),
    (getpstr, "description"),
    (getostype, "suite code"),
    (getostype, "event code"),
    (getdata, "returns"),
    (getdata, "accepts"),
    (getlist, "optional arguments", getargument)
    ]
getproperty = [
    (getpstr, "name"),
    (getostype, "code"),
    (getdata, "what")
    ]
getelement = [
    (getostype, "type"),
    (getlist, "keyform", getostype)
    ]
getclass = [
    (getpstr, "name"),
    (getostype, "class code"),
    (getpstr, "description"),
    (getlist, "properties", getproperty),
    (getlist, "elements", getelement)
    ]
getcomparison = [
    (getpstr, "operator name"),
    (getostype, "operator ID"),
    (getpstr, "operator comment"),
    ]
getenumerator = [
    (getpstr, "enumerator name"),
    (getostype, "enumerator ID"),
    (getpstr, "enumerator comment")
    ]
getenumeration = [
    (getostype, "enumeration ID"),
    (getlist, "enumerator", getenumerator)
    ]
getsuite = [
    (getpstr, "suite name"),
    (getpstr, "suite description"),
    (getostype, "suite ID"),
    (getword, "suite level"),
    (getword, "suite version"),
    (getlist, "events", getevent),
    (getlist, "classes", getclass),
    (getlist, "comparisons", getcomparison),
    (getlist, "enumerations", getenumeration)
    ]
getaete = [
    (getword, "major/minor version in BCD"),
    (getword, "language code"),
    (getword, "script code"),
    (getlist, "suites", getsuite)
    ]

def compileaete(aete, resinfo, fname, output=None, basepkgname=None,
        edit_modnames=None, creatorsignature=None, verbose=None):
    """Generate code for a full aete resource. fname passed for doc purposes"""
    [version, language, script, suites] = aete
    major, minor = divmod(version, 256)
    if not creatorsignature:
        creatorsignature, dummy = MacOS.GetCreatorAndType(fname)
    packagename = identify(os.path.splitext(os.path.basename(fname))[0])
    if language:
        packagename = packagename+'_lang%d'%language
    if script:
        packagename = packagename+'_script%d'%script
    if len(packagename) > 27:
        packagename = packagename[:27]
    if output:
        # XXXX Put this in site-packages if it isn't a full pathname?
        if not os.path.exists(output):
            os.mkdir(output)
        pathname = output
    else:
        pathname = EasyDialogs.AskFolder(message='Create and select package folder for %s'%packagename,
            defaultLocation=DEFAULT_USER_PACKAGEFOLDER)
        output = pathname
    if not pathname:
        return
    packagename = os.path.split(os.path.normpath(pathname))[1]
    if not basepkgname:
        basepkgname = EasyDialogs.AskFolder(message='Package folder for base suite (usually StdSuites)',
            defaultLocation=DEFAULT_STANDARD_PACKAGEFOLDER)
    if basepkgname:
        dirname, basepkgname = os.path.split(os.path.normpath(basepkgname))
        if dirname and not dirname in sys.path:
            sys.path.insert(0, dirname)
        basepackage = __import__(basepkgname)
    else:
        basepackage = None
    suitelist = []
    allprecompinfo = []
    allsuites = []
    for suite in suites:
        compiler = SuiteCompiler(suite, basepackage, output, edit_modnames, verbose)
        code, modname, precompinfo = compiler.precompilesuite()
        if not code:
            continue
        allprecompinfo = allprecompinfo + precompinfo
        suiteinfo = suite, pathname, modname
        suitelist.append((code, modname))
        allsuites.append(compiler)
    for compiler in allsuites:
        compiler.compilesuite(major, minor, language, script, fname, allprecompinfo)
    initfilename = os.path.join(output, '__init__.py')
    fp = open(initfilename, 'w')
    MacOS.SetCreatorAndType(initfilename, 'Pyth', 'TEXT')
    fp.write('"""\n')
    fp.write("Package generated from %s\n"%ascii(fname))
    if resinfo:
        fp.write("Resource %s resid %d %s\n"%(ascii(resinfo[1]), resinfo[0], ascii(resinfo[2])))
    fp.write('"""\n')
    fp.write('import aetools\n')
    fp.write('Error = aetools.Error\n')
    suitelist.sort()
    for code, modname in suitelist:
        fp.write("import %s\n" % modname)
    fp.write("\n\n_code_to_module = {\n")
    for code, modname in suitelist:
        fp.write("    '%s' : %s,\n"%(ascii(code), modname))
    fp.write("}\n\n")
    fp.write("\n\n_code_to_fullname = {\n")
    for code, modname in suitelist:
        fp.write("    '%s' : ('%s.%s', '%s'),\n"%(ascii(code), packagename, modname, modname))
    fp.write("}\n\n")
    for code, modname in suitelist:
        fp.write("from %s import *\n"%modname)

    # Generate property dicts and element dicts for all types declared in this module
    fp.write("\ndef getbaseclasses(v):\n")
    fp.write("    if not getattr(v, '_propdict', None):\n")
    fp.write("        v._propdict = {}\n")
    fp.write("        v._elemdict = {}\n")
    fp.write("        for superclassname in getattr(v, '_superclassnames', []):\n")
    fp.write("            superclass = eval(superclassname)\n")
    fp.write("            getbaseclasses(superclass)\n")
    fp.write("            v._propdict.update(getattr(superclass, '_propdict', {}))\n")
    fp.write("            v._elemdict.update(getattr(superclass, '_elemdict', {}))\n")
    fp.write("        v._propdict.update(getattr(v, '_privpropdict', {}))\n")
    fp.write("        v._elemdict.update(getattr(v, '_privelemdict', {}))\n")
    fp.write("\n")
    fp.write("import StdSuites\n")
    allprecompinfo.sort()
    if allprecompinfo:
        fp.write("\n#\n# Set property and element dictionaries now that all classes have been defined\n#\n")
        for codenamemapper in allprecompinfo:
            for k, v in codenamemapper.getall('class'):
                fp.write("getbaseclasses(%s)\n" % v)

    # Generate a code-to-name mapper for all of the types (classes) declared in this module
    application_class = None
    if allprecompinfo:
        fp.write("\n#\n# Indices of types declared in this module\n#\n")
        fp.write("_classdeclarations = {\n")
        for codenamemapper in allprecompinfo:
            for k, v in codenamemapper.getall('class'):
                fp.write("    %r : %s,\n" % (k, v))
                if k == 'capp':
                    application_class = v
        fp.write("}\n")


    if suitelist:
        fp.write("\n\nclass %s(%s_Events"%(packagename, suitelist[0][1]))
        for code, modname in suitelist[1:]:
            fp.write(",\n        %s_Events"%modname)
        fp.write(",\n        aetools.TalkTo):\n")
        fp.write("    _signature = %r\n\n"%(creatorsignature,))
        fp.write("    _moduleName = '%s'\n\n"%packagename)
        if application_class:
            fp.write("    _elemdict = %s._elemdict\n" % application_class)
            fp.write("    _propdict = %s._propdict\n" % application_class)
    fp.close()

class SuiteCompiler:
    def __init__(self, suite, basepackage, output, edit_modnames, verbose):
        self.suite = suite
        self.basepackage = basepackage
        self.edit_modnames = edit_modnames
        self.output = output
        self.verbose = verbose

        # Set by precompilesuite
        self.pathname = None
        self.modname = None

        # Set by compilesuite
        self.fp = None
        self.basemodule = None
        self.enumsneeded = {}

    def precompilesuite(self):
        """Parse a single suite without generating the output. This step is needed
        so we can resolve recursive references by suites to enums/comps/etc declared
        in other suites"""
        [name, desc, code, level, version, events, classes, comps, enums] = self.suite

        modname = identify(name)
        if len(modname) > 28:
            modname = modname[:27]
        if self.edit_modnames is None:
            self.pathname = EasyDialogs.AskFileForSave(message='Python output file',
                savedFileName=modname+'.py')
        else:
            for old, new in self.edit_modnames:
                if old == modname:
                    modname = new
            if modname:
                self.pathname = os.path.join(self.output, modname + '.py')
            else:
                self.pathname = None
        if not self.pathname:
            return None, None, None

        self.modname = os.path.splitext(os.path.split(self.pathname)[1])[0]

        if self.basepackage and code in self.basepackage._code_to_module:
            # We are an extension of a baseclass (usually an application extending
            # Standard_Suite or so). Import everything from our base module
            basemodule = self.basepackage._code_to_module[code]
        else:
            # We are not an extension.
            basemodule = None

        self.enumsneeded = {}
        for event in events:
            self.findenumsinevent(event)

        objc = ObjectCompiler(None, self.modname, basemodule, interact=(self.edit_modnames is None),
            verbose=self.verbose)
        for cls in classes:
            objc.compileclass(cls)
        for cls in classes:
            objc.fillclasspropsandelems(cls)
        for comp in comps:
            objc.compilecomparison(comp)
        for enum in enums:
            objc.compileenumeration(enum)

        for enum in self.enumsneeded.keys():
            objc.checkforenum(enum)

        objc.dumpindex()

        precompinfo = objc.getprecompinfo(self.modname)

        return code, self.modname, precompinfo

    def compilesuite(self, major, minor, language, script, fname, precompinfo):
        """Generate code for a single suite"""
        [name, desc, code, level, version, events, classes, comps, enums] = self.suite
        # Sort various lists, so re-generated source is easier compared
        def class_sorter(k1, k2):
            """Sort classes by code, and make sure main class sorts before synonyms"""
            # [name, code, desc, properties, elements] = cls
            if k1[1] < k2[1]: return -1
            if k1[1] > k2[1]: return 1
            if not k2[3] or k2[3][0][1] == 'c@#!':
                # This is a synonym, the other one is better
                return -1
            if not k1[3] or k1[3][0][1] == 'c@#!':
                # This is a synonym, the other one is better
                return 1
            return 0

        events.sort()
        classes.sort(class_sorter)
        comps.sort()
        enums.sort()

        self.fp = fp = open(self.pathname, 'w')
        MacOS.SetCreatorAndType(self.pathname, 'Pyth', 'TEXT')

        fp.write('"""Suite %s: %s\n' % (ascii(name), ascii(desc)))
        fp.write("Level %d, version %d\n\n" % (level, version))
        fp.write("Generated from %s\n"%ascii(fname))
        fp.write("AETE/AEUT resource version %d/%d, language %d, script %d\n" % \
            (major, minor, language, script))
        fp.write('"""\n\n')

        fp.write('import aetools\n')
        fp.write('import MacOS\n\n')
        fp.write("_code = %r\n\n"% (code,))
        if self.basepackage and code in self.basepackage._code_to_module:
            # We are an extension of a baseclass (usually an application extending
            # Standard_Suite or so). Import everything from our base module
            fp.write('from %s import *\n'%self.basepackage._code_to_fullname[code][0])
            basemodule = self.basepackage._code_to_module[code]
        elif self.basepackage and code.lower() in self.basepackage._code_to_module:
            # This is needed by CodeWarrior and some others.
            fp.write('from %s import *\n'%self.basepackage._code_to_fullname[code.lower()][0])
            basemodule = self.basepackage._code_to_module[code.lower()]
        else:
            # We are not an extension.
            basemodule = None
        self.basemodule = basemodule
        self.compileclassheader()

        self.enumsneeded = {}
        if events:
            for event in events:
                self.compileevent(event)
        else:
            fp.write("    pass\n\n")

        objc = ObjectCompiler(fp, self.modname, basemodule, precompinfo, interact=(self.edit_modnames is None),
            verbose=self.verbose)
        for cls in classes:
            objc.compileclass(cls)
        for cls in classes:
            objc.fillclasspropsandelems(cls)
        for comp in comps:
            objc.compilecomparison(comp)
        for enum in enums:
            objc.compileenumeration(enum)

        for enum in self.enumsneeded.keys():
            objc.checkforenum(enum)

        objc.dumpindex()

    def compileclassheader(self):
        """Generate class boilerplate"""
        classname = '%s_Events'%self.modname
        if self.basemodule:
            modshortname = string.split(self.basemodule.__name__, '.')[-1]
            baseclassname = '%s_Events'%modshortname
            self.fp.write("class %s(%s):\n\n"%(classname, baseclassname))
        else:
            self.fp.write("class %s:\n\n"%classname)

    def compileevent(self, event):
        """Generate code for a single event"""
        [name, desc, code, subcode, returns, accepts, arguments] = event
        fp = self.fp
        funcname = identify(name)
        #
        # generate name->keyword map
        #
        if arguments:
            fp.write("    _argmap_%s = {\n"%funcname)
            for a in arguments:
                fp.write("        %r : %r,\n"%(identify(a[0]), a[1]))
            fp.write("    }\n\n")

        #
        # Generate function header
        #
        has_arg = (not is_null(accepts))
        opt_arg = (has_arg and is_optional(accepts))

        fp.write("    def %s(self, "%funcname)
        if has_arg:
            if not opt_arg:
                fp.write("_object, ")       # Include direct object, if it has one
            else:
                fp.write("_object=None, ")  # Also include if it is optional
        else:
            fp.write("_no_object=None, ")   # For argument checking
        fp.write("_attributes={}, **_arguments):\n")    # include attribute dict and args
        #
        # Generate doc string (important, since it may be the only
        # available documentation, due to our name-remaping)
        #
        fp.write('        """%s: %s\n'%(ascii(name), ascii(desc)))
        if has_arg:
            fp.write("        Required argument: %s\n"%getdatadoc(accepts))
        elif opt_arg:
            fp.write("        Optional argument: %s\n"%getdatadoc(accepts))
        for arg in arguments:
            fp.write("        Keyword argument %s: %s\n"%(identify(arg[0]),
                    getdatadoc(arg[2])))
        fp.write("        Keyword argument _attributes: AppleEvent attribute dictionary\n")
        if not is_null(returns):
            fp.write("        Returns: %s\n"%getdatadoc(returns))
        fp.write('        """\n')
        #
        # Fiddle the args so everything ends up in 'arguments' dictionary
        #
        fp.write("        _code = %r\n"% (code,))
        fp.write("        _subcode = %r\n\n"% (subcode,))
        #
        # Do keyword name substitution
        #
        if arguments:
            fp.write("        aetools.keysubst(_arguments, self._argmap_%s)\n"%funcname)
        else:
            fp.write("        if _arguments: raise TypeError, 'No optional args expected'\n")
        #
        # Stuff required arg (if there is one) into arguments
        #
        if has_arg:
            fp.write("        _arguments['----'] = _object\n")
        elif opt_arg:
            fp.write("        if _object:\n")
            fp.write("            _arguments['----'] = _object\n")
        else:
            fp.write("        if _no_object is not None: raise TypeError, 'No direct arg expected'\n")
        fp.write("\n")
        #
        # Do enum-name substitution
        #
        for a in arguments:
            if is_enum(a[2]):
                kname = a[1]
                ename = a[2][0]
                if ename != '****':
                    fp.write("        aetools.enumsubst(_arguments, %r, _Enum_%s)\n" %
                        (kname, identify(ename)))
                    self.enumsneeded[ename] = 1
        fp.write("\n")
        #
        # Do the transaction
        #
        fp.write("        _reply, _arguments, _attributes = self.send(_code, _subcode,\n")
        fp.write("                _arguments, _attributes)\n")
        #
        # Error handling
        #
        fp.write("        if _arguments.get('errn', 0):\n")
        fp.write("            raise aetools.Error, aetools.decodeerror(_arguments)\n")
        fp.write("        # XXXX Optionally decode result\n")
        #
        # Decode result
        #
        fp.write("        if '----' in _arguments:\n")
        if is_enum(returns):
            fp.write("            # XXXX Should do enum remapping here...\n")
        fp.write("            return _arguments['----']\n")
        fp.write("\n")

    def findenumsinevent(self, event):
        """Find all enums for a single event"""
        [name, desc, code, subcode, returns, accepts, arguments] = event
        for a in arguments:
            if is_enum(a[2]):
                ename = a[2][0]
                if ename != '****':
                    self.enumsneeded[ename] = 1

#
# This class stores the code<->name translations for a single module. It is used
# to keep the information while we're compiling the module, but we also keep these objects
# around so if one suite refers to, say, an enum in another suite we know where to
# find it. Finally, if we really can't find a code, the user can add modules by
# hand.
#
class CodeNameMapper:

    def __init__(self, interact=1, verbose=None):
        self.code2name = {
            "property" : {},
            "class" : {},
            "enum" : {},
            "comparison" : {},
        }
        self.name2code =  {
            "property" : {},
            "class" : {},
            "enum" : {},
            "comparison" : {},
        }
        self.modulename = None
        self.star_imported = 0
        self.can_interact = interact
        self.verbose = verbose

    def addnamecode(self, type, name, code):
        self.name2code[type][name] = code
        if code not in self.code2name[type]:
            self.code2name[type][code] = name

    def hasname(self, name):
        for dict in self.name2code.values():
            if name in dict:
                return True
        return False

    def hascode(self, type, code):
        return code in self.code2name[type]

    def findcodename(self, type, code):
        if not self.hascode(type, code):
            return None, None, None
        name = self.code2name[type][code]
        if self.modulename and not self.star_imported:
            qualname = '%s.%s'%(self.modulename, name)
        else:
            qualname = name
        return name, qualname, self.modulename

    def getall(self, type):
        return self.code2name[type].items()

    def addmodule(self, module, name, star_imported):
        self.modulename = name
        self.star_imported = star_imported
        for code, name in module._propdeclarations.items():
            self.addnamecode('property', name, code)
        for code, name in module._classdeclarations.items():
            self.addnamecode('class', name, code)
        for code in module._enumdeclarations.keys():
            self.addnamecode('enum', '_Enum_'+identify(code), code)
        for code, name in module._compdeclarations.items():
            self.addnamecode('comparison', name, code)

    def prepareforexport(self, name=None):
        if not self.modulename:
            self.modulename = name
        return self

class ObjectCompiler:
    def __init__(self, fp, modname, basesuite, othernamemappers=None, interact=1,
            verbose=None):
        self.fp = fp
        self.verbose = verbose
        self.basesuite = basesuite
        self.can_interact = interact
        self.modulename = modname
        self.namemappers = [CodeNameMapper(self.can_interact, self.verbose)]
        if othernamemappers:
            self.othernamemappers = othernamemappers[:]
        else:
            self.othernamemappers = []
        if basesuite:
            basemapper = CodeNameMapper(self.can_interact, self.verbose)
            basemapper.addmodule(basesuite, '', 1)
            self.namemappers.append(basemapper)

    def getprecompinfo(self, modname):
        list = []
        for mapper in self.namemappers:
            emapper = mapper.prepareforexport(modname)
            if emapper:
                list.append(emapper)
        return list

    def findcodename(self, type, code):
        while 1:
            # First try: check whether we already know about this code.
            for mapper in self.namemappers:
                if mapper.hascode(type, code):
                    return mapper.findcodename(type, code)
            # Second try: maybe one of the other modules knows about it.
            for mapper in self.othernamemappers:
                if mapper.hascode(type, code):
                    self.othernamemappers.remove(mapper)
                    self.namemappers.append(mapper)
                    if self.fp:
                        self.fp.write("import %s\n"%mapper.modulename)
                    break
            else:
                # If all this has failed we ask the user for a guess on where it could
                # be and retry.
                if self.fp:
                    m = self.askdefinitionmodule(type, code)
                else:
                    m = None
                if not m: return None, None, None
                mapper = CodeNameMapper(self.can_interact, self.verbose)
                mapper.addmodule(m, m.__name__, 0)
                self.namemappers.append(mapper)

    def hasname(self, name):
        for mapper in self.othernamemappers:
            if mapper.hasname(name) and mapper.modulename != self.modulename:
                if self.verbose:
                    print >>self.verbose, "Duplicate Python identifier:", name, self.modulename, mapper.modulename
                return True
        return False

    def askdefinitionmodule(self, type, code):
        if not self.can_interact:
            if self.verbose:
                print >>self.verbose, "** No definition for %s '%s' found" % (type, code)
            return None
        path = EasyDialogs.AskFileForSave(message='Where is %s %s declared?'%(type, code))
        if not path: return
        path, file = os.path.split(path)
        modname = os.path.splitext(file)[0]
        if not path in sys.path:
            sys.path.insert(0, path)
        m = __import__(modname)
        self.fp.write("import %s\n"%modname)
        return m

    def compileclass(self, cls):
        [name, code, desc, properties, elements] = cls
        pname = identify(name)
        if self.namemappers[0].hascode('class', code):
            # plural forms and such
            othername, dummy, dummy = self.namemappers[0].findcodename('class', code)
            if self.fp:
                self.fp.write("\n%s = %s\n"%(pname, othername))
        else:
            if self.fp:
                self.fp.write('\nclass %s(aetools.ComponentItem):\n' % pname)
                self.fp.write('    """%s - %s """\n' % (ascii(name), ascii(desc)))
                self.fp.write('    want = %r\n' % (code,))
        self.namemappers[0].addnamecode('class', pname, code)
        is_application_class = (code == 'capp')
        properties.sort()
        for prop in properties:
            self.compileproperty(prop, is_application_class)
        elements.sort()
        for elem in elements:
            self.compileelement(elem)

    def compileproperty(self, prop, is_application_class=False):
        [name, code, what] = prop
        if code == 'c@#!':
            # Something silly with plurals. Skip it.
            return
        pname = identify(name)
        if self.namemappers[0].hascode('property', code):
            # plural forms and such
            othername, dummy, dummy = self.namemappers[0].findcodename('property', code)
            if pname == othername:
                return
            if self.fp:
                self.fp.write("\n_Prop_%s = _Prop_%s\n"%(pname, othername))
        else:
            if self.fp:
                self.fp.write("class _Prop_%s(aetools.NProperty):\n" % pname)
                self.fp.write('    """%s - %s """\n' % (ascii(name), ascii(what[1])))
                self.fp.write("    which = %r\n" % (code,))
                self.fp.write("    want = %r\n" % (what[0],))
        self.namemappers[0].addnamecode('property', pname, code)
        if is_application_class and self.fp:
            self.fp.write("%s = _Prop_%s()\n" % (pname, pname))

    def compileelement(self, elem):
        [code, keyform] = elem
        if self.fp:
            self.fp.write("#        element %r as %s\n" % (code, keyform))

    def fillclasspropsandelems(self, cls):
        [name, code, desc, properties, elements] = cls
        cname = identify(name)
        if self.namemappers[0].hascode('class', code) and \
                self.namemappers[0].findcodename('class', code)[0] != cname:
            # This is an other name (plural or so) for something else. Skip.
            if self.fp and (elements or len(properties) > 1 or (len(properties) == 1 and
                properties[0][1] != 'c@#!')):
                if self.verbose:
                    print >>self.verbose, '** Skip multiple %s of %s (code %r)' % (cname, self.namemappers[0].findcodename('class', code)[0], code)
                raise RuntimeError, "About to skip non-empty class"
            return
        plist = []
        elist = []
        superclasses = []
        for prop in properties:
            [pname, pcode, what] = prop
            if pcode == "c@#^":
                superclasses.append(what)
            if pcode == 'c@#!':
                continue
            pname = identify(pname)
            plist.append(pname)

        superclassnames = []
        for superclass in superclasses:
            superId, superDesc, dummy = superclass
            superclassname, fullyqualifiedname, module = self.findcodename("class", superId)
            # I don't think this is correct:
            if superclassname == cname:
                pass # superclassnames.append(fullyqualifiedname)
            else:
                superclassnames.append(superclassname)

        if self.fp:
            self.fp.write("%s._superclassnames = %r\n"%(cname, superclassnames))

        for elem in elements:
            [ecode, keyform] = elem
            if ecode == 'c@#!':
                continue
            name, ename, module = self.findcodename('class', ecode)
            if not name:
                if self.fp:
                    self.fp.write("# XXXX %s element %r not found!!\n"%(cname, ecode))
            else:
                elist.append((name, ename))

        plist.sort()
        elist.sort()

        if self.fp:
            self.fp.write("%s._privpropdict = {\n"%cname)
            for n in plist:
                self.fp.write("    '%s' : _Prop_%s,\n"%(n, n))
            self.fp.write("}\n")
            self.fp.write("%s._privelemdict = {\n"%cname)
            for n, fulln in elist:
                self.fp.write("    '%s' : %s,\n"%(n, fulln))
            self.fp.write("}\n")

    def compilecomparison(self, comp):
        [name, code, comment] = comp
        iname = identify(name)
        self.namemappers[0].addnamecode('comparison', iname, code)
        if self.fp:
            self.fp.write("class %s(aetools.NComparison):\n" % iname)
            self.fp.write('    """%s - %s """\n' % (ascii(name), ascii(comment)))

    def compileenumeration(self, enum):
        [code, items] = enum
        name = "_Enum_%s" % identify(code)
        if self.fp:
            self.fp.write("%s = {\n" % name)
            for item in items:
                self.compileenumerator(item)
            self.fp.write("}\n\n")
        self.namemappers[0].addnamecode('enum', name, code)
        return code

    def compileenumerator(self, item):
        [name, code, desc] = item
        self.fp.write("    %r : %r,\t# %s\n" % (identify(name), code, ascii(desc)))

    def checkforenum(self, enum):
        """This enum code is used by an event. Make sure it's available"""
        name, fullname, module = self.findcodename('enum', enum)
        if not name:
            if self.fp:
                self.fp.write("_Enum_%s = None # XXXX enum %s not found!!\n"%(identify(enum), ascii(enum)))
            return
        if module:
            if self.fp:
                self.fp.write("from %s import %s\n"%(module, name))

    def dumpindex(self):
        if not self.fp:
            return
        self.fp.write("\n#\n# Indices of types declared in this module\n#\n")

        self.fp.write("_classdeclarations = {\n")
        classlist = self.namemappers[0].getall('class')
        classlist.sort()
        for k, v in classlist:
            self.fp.write("    %r : %s,\n" % (k, v))
        self.fp.write("}\n")

        self.fp.write("\n_propdeclarations = {\n")
        proplist = self.namemappers[0].getall('property')
        proplist.sort()
        for k, v in proplist:
            self.fp.write("    %r : _Prop_%s,\n" % (k, v))
        self.fp.write("}\n")

        self.fp.write("\n_compdeclarations = {\n")
        complist = self.namemappers[0].getall('comparison')
        complist.sort()
        for k, v in complist:
            self.fp.write("    %r : %s,\n" % (k, v))
        self.fp.write("}\n")

        self.fp.write("\n_enumdeclarations = {\n")
        enumlist = self.namemappers[0].getall('enum')
        enumlist.sort()
        for k, v in enumlist:
            self.fp.write("    %r : %s,\n" % (k, v))
        self.fp.write("}\n")

def compiledata(data):
    [type, description, flags] = data
    return "%r -- %r %s" % (type, description, compiledataflags(flags))

def is_null(data):
    return data[0] == 'null'

def is_optional(data):
    return (data[2] & 0x8000)

def is_enum(data):
    return (data[2] & 0x2000)

def getdatadoc(data):
    [type, descr, flags] = data
    if descr:
        return ascii(descr)
    if type == '****':
        return 'anything'
    if type == 'obj ':
        return 'an AE object reference'
    return "undocumented, typecode %r"%(type,)

dataflagdict = {15: "optional", 14: "list", 13: "enum", 12: "mutable"}
def compiledataflags(flags):
    bits = []
    for i in range(16):
        if flags & (1<<i):
            if i in dataflagdict.keys():
                bits.append(dataflagdict[i])
            else:
                bits.append(repr(i))
    return '[%s]' % string.join(bits)

def ascii(str):
    """Return a string with all non-ascii characters hex-encoded"""
    if type(str) != type(''):
        return map(ascii, str)
    rv = ''
    for c in str:
        if c in ('\t', '\n', '\r') or ' ' <= c < chr(0x7f):
            rv = rv + c
        else:
            rv = rv + '\\' + 'x%02.2x' % ord(c)
    return rv

def identify(str):
    """Turn any string into an identifier:
    - replace space by _
    - replace other illegal chars by _xx_ (hex code)
    - append _ if the result is a python keyword
    """
    if not str:
        return "empty_ae_name_"
    rv = ''
    ok = string.ascii_letters + '_'
    ok2 = ok + string.digits
    for c in str:
        if c in ok:
            rv = rv + c
        elif c == ' ':
            rv = rv + '_'
        else:
            rv = rv + '_%02.2x_'%ord(c)
        ok = ok2
    if keyword.iskeyword(rv):
        rv = rv + '_'
    return rv

# Call the main program

if __name__ == '__main__':
    main()
    sys.exit(1)



"""
Package generated from /Volumes/Sap/Applications (Mac OS 9)/Metrowerks CodeWarrior 7.0/Metrowerks CodeWarrior/CodeWarrior IDE 4.2.5
"""

from warnings import warnpy3k
warnpy3k("In 3.x, the CodeWarrior package is removed.", stacklevel=2)

import aetools
Error = aetools.Error
import CodeWarrior_suite
import Standard_Suite
import Metrowerks_Shell_Suite
import Required


_code_to_module = {
    'CWIE' : CodeWarrior_suite,
    'CoRe' : Standard_Suite,
    'MMPR' : Metrowerks_Shell_Suite,
    'reqd' : Required,
}



_code_to_fullname = {
    'CWIE' : ('CodeWarrior.CodeWarrior_suite', 'CodeWarrior_suite'),
    'CoRe' : ('CodeWarrior.Standard_Suite', 'Standard_Suite'),
    'MMPR' : ('CodeWarrior.Metrowerks_Shell_Suite', 'Metrowerks_Shell_Suite'),
    'reqd' : ('CodeWarrior.Required', 'Required'),
}

from CodeWarrior_suite import *
from Standard_Suite import *
from Metrowerks_Shell_Suite import *
from Required import *

def getbaseclasses(v):
    if not getattr(v, '_propdict', None):
        v._propdict = {}
        v._elemdict = {}
        for superclassname in getattr(v, '_superclassnames', []):
            superclass = eval(superclassname)
            getbaseclasses(superclass)
            v._propdict.update(getattr(superclass, '_propdict', {}))
            v._elemdict.update(getattr(superclass, '_elemdict', {}))
        v._propdict.update(getattr(v, '_privpropdict', {}))
        v._elemdict.update(getattr(v, '_privelemdict', {}))

import StdSuites

#
# Set property and element dictionaries now that all classes have been defined
#
getbaseclasses(character)
getbaseclasses(selection_2d_object)
getbaseclasses(application)
getbaseclasses(document)
getbaseclasses(text)
getbaseclasses(window)
getbaseclasses(file)
getbaseclasses(line)
getbaseclasses(insertion_point)
getbaseclasses(single_class_browser)
getbaseclasses(project_document)
getbaseclasses(symbol_browser)
getbaseclasses(editor_document)
getbaseclasses(file_compare_document)
getbaseclasses(class_browser)
getbaseclasses(subtarget)
getbaseclasses(message_document)
getbaseclasses(project_inspector)
getbaseclasses(text_document)
getbaseclasses(catalog_document)
getbaseclasses(class_hierarchy)
getbaseclasses(target)
getbaseclasses(build_progress_document)
getbaseclasses(target_file)
getbaseclasses(ToolServer_worksheet)
getbaseclasses(single_class_hierarchy)
getbaseclasses(File_Mapping)
getbaseclasses(browser_catalog)
getbaseclasses(Build_Settings)
getbaseclasses(ProjectFile)
getbaseclasses(VCS_Setup)
getbaseclasses(data_member)
getbaseclasses(Shielded_Folder)
getbaseclasses(Custom_Keywords)
getbaseclasses(Path_Information)
getbaseclasses(Segment)
getbaseclasses(Source_Tree)
getbaseclasses(Access_Paths)
getbaseclasses(Debugger_Windowing)
getbaseclasses(Relative_Path)
getbaseclasses(Environment_Variable)
getbaseclasses(base_class)
getbaseclasses(Debugger_Display)
getbaseclasses(Build_Extras)
getbaseclasses(Error_Information)
getbaseclasses(Editor)
getbaseclasses(Shielded_Folders)
getbaseclasses(Extras)
getbaseclasses(File_Mappings)
getbaseclasses(Function_Information)
getbaseclasses(Debugger_Target)
getbaseclasses(Syntax_Coloring)
getbaseclasses(class_)
getbaseclasses(Global_Source_Trees)
getbaseclasses(Target_Settings)
getbaseclasses(Debugger_Global)
getbaseclasses(member_function)
getbaseclasses(Runtime_Settings)
getbaseclasses(Plugin_Settings)
getbaseclasses(Browser_Coloring)
getbaseclasses(Font)
getbaseclasses(Target_Source_Trees)

#
# Indices of types declared in this module
#
_classdeclarations = {
    'cha ' : character,
    'csel' : selection_2d_object,
    'capp' : application,
    'docu' : document,
    'ctxt' : text,
    'cwin' : window,
    'file' : file,
    'clin' : line,
    'cins' : insertion_point,
    '1BRW' : single_class_browser,
    'PRJD' : project_document,
    'SYMB' : symbol_browser,
    'EDIT' : editor_document,
    'COMP' : file_compare_document,
    'BROW' : class_browser,
    'SBTG' : subtarget,
    'MSSG' : message_document,
    'INSP' : project_inspector,
    'TXTD' : text_document,
    'CTLG' : catalog_document,
    'HIER' : class_hierarchy,
    'TRGT' : target,
    'PRGS' : build_progress_document,
    'SRCF' : target_file,
    'TOOL' : ToolServer_worksheet,
    '1HIR' : single_class_hierarchy,
    'FMap' : File_Mapping,
    'Cata' : browser_catalog,
    'BSTG' : Build_Settings,
    'SrcF' : ProjectFile,
    'VCSs' : VCS_Setup,
    'DtMb' : data_member,
    'SFit' : Shielded_Folder,
    'CUKW' : Custom_Keywords,
    'PInf' : Path_Information,
    'Seg ' : Segment,
    'SrcT' : Source_Tree,
    'PATH' : Access_Paths,
    'DbWN' : Debugger_Windowing,
    'RlPt' : Relative_Path,
    'EnvV' : Environment_Variable,
    'BsCl' : base_class,
    'DbDS' : Debugger_Display,
    'LXTR' : Build_Extras,
    'ErrM' : Error_Information,
    'EDTR' : Editor,
    'SHFL' : Shielded_Folders,
    'GXTR' : Extras,
    'FLMP' : File_Mappings,
    'FDef' : Function_Information,
    'DbTG' : Debugger_Target,
    'SNTX' : Syntax_Coloring,
    'Clas' : class_,
    'GSTs' : Global_Source_Trees,
    'TARG' : Target_Settings,
    'DbGL' : Debugger_Global,
    'MbFn' : member_function,
    'RSTG' : Runtime_Settings,
    'PSTG' : Plugin_Settings,
    'BRKW' : Browser_Coloring,
    'mFNT' : Font,
    'TSTs' : Target_Source_Trees,
}


class CodeWarrior(CodeWarrior_suite_Events,
        Standard_Suite_Events,
        Metrowerks_Shell_Suite_Events,
        Required_Events,
        aetools.TalkTo):
    _signature = 'CWIE'

    _moduleName = 'CodeWarrior'



#
# cl.h - Compression Library typedefs and prototypes
#
#   01/07/92    Cleanup by Brian Knittel
#   02/18/92    Original Version by Brian Knittel
#

#
# originalFormat parameter values
#
from warnings import warnpy3k
warnpy3k("the CL_old module has been removed in Python 3.0", stacklevel=2)
del warnpy3k

MAX_NUMBER_OF_ORIGINAL_FORMATS = 32

# Audio
MONO = 0
STEREO_INTERLEAVED = 1

# Video
# YUV is defined to be the same thing as YCrCb (luma and two chroma components).
# 422 is appended to YUV (or YCrCb) if the chroma is sub-sampled by 2
#       horizontally, packed as U Y1 V Y2 (byte order).
# 422HC is appended to YUV (or YCrCb) if the chroma is sub-sampled by 2
#       vertically in addition to horizontally, and is packed the same as
#       422 except that U & V are not valid on the second line.
#
RGB = 0
RGBX = 1
RGBA = 2
RGB332 = 3

GRAYSCALE = 4
Y = 4
YUV = 5
YCbCr = 5
YUV422 = 6                              # 4:2:2 sampling
YCbCr422 = 6                            # 4:2:2 sampling
YUV422HC = 7                            # 4:1:1 sampling
YCbCr422HC = 7                          # 4:1:1 sampling
YUV422DC = 7                            # 4:1:1 sampling
YCbCr422DC = 7                          # 4:1:1 sampling

BEST_FIT = -1

def BytesPerSample(s):
    if s in (MONO, YUV):
        return 2
    elif s == STEREO_INTERLEAVED:
        return 4
    else:
        return 0

def BytesPerPixel(f):
    if f in (RGB, YUV):
        return 3
    elif f in (RGBX, RGBA):
        return 4
    elif f in (RGB332, GRAYSCALE):
        return 1
    else:
        return 2

def AudioFormatName(f):
    if f == MONO:
        return 'MONO'
    elif f == STEREO_INTERLEAVED:
        return 'STEREO_INTERLEAVED'
    else:
        return 'Not a valid format'

def VideoFormatName(f):
    if f == RGB:
        return 'RGB'
    elif f == RGBX:
        return 'RGBX'
    elif f == RGBA:
        return 'RGBA'
    elif f == RGB332:
        return 'RGB332'
    elif f == GRAYSCALE:
        return 'GRAYSCALE'
    elif f == YUV:
        return 'YUV'
    elif f == YUV422:
        return 'YUV422'
    elif f == YUV422DC:
        return 'YUV422DC'
    else:
        return 'Not a valid format'

MAX_NUMBER_OF_AUDIO_ALGORITHMS = 32
MAX_NUMBER_OF_VIDEO_ALGORITHMS = 32

#
# Algorithm types
#
AUDIO = 0
VIDEO = 1

def AlgorithmNumber(scheme):
    return scheme & 0x7fff
def AlgorithmType(scheme):
    return (scheme >> 15) & 1
def Algorithm(type, n):
    return n | ((type & 1) << 15)

#
# "compressionScheme" argument values
#
UNKNOWN_SCHEME = -1

UNCOMPRESSED_AUDIO = Algorithm(AUDIO, 0)
G711_ULAW = Algorithm(AUDIO, 1)
ULAW = Algorithm(AUDIO, 1)
G711_ALAW = Algorithm(AUDIO, 2)
ALAW = Algorithm(AUDIO, 2)
AWARE_MPEG_AUDIO = Algorithm(AUDIO, 3)
AWARE_MULTIRATE = Algorithm(AUDIO, 4)

UNCOMPRESSED = Algorithm(VIDEO, 0)
UNCOMPRESSED_VIDEO = Algorithm(VIDEO, 0)
RLE = Algorithm(VIDEO, 1)
JPEG = Algorithm(VIDEO, 2)
MPEG_VIDEO = Algorithm(VIDEO, 3)
MVC1 = Algorithm(VIDEO, 4)
RTR = Algorithm(VIDEO, 5)
RTR1 = Algorithm(VIDEO, 5)

#
# Parameters
#
MAX_NUMBER_OF_PARAMS = 256
# Default Parameters
IMAGE_WIDTH = 0
IMAGE_HEIGHT = 1
ORIGINAL_FORMAT = 2
INTERNAL_FORMAT = 3
COMPONENTS = 4
BITS_PER_COMPONENT = 5
FRAME_RATE = 6
COMPRESSION_RATIO = 7
EXACT_COMPRESSION_RATIO = 8
FRAME_BUFFER_SIZE = 9
COMPRESSED_BUFFER_SIZE = 10
BLOCK_SIZE = 11
PREROLL = 12
FRAME_TYPE = 13
ALGORITHM_ID = 14
ALGORITHM_VERSION = 15
ORIENTATION = 16
NUMBER_OF_FRAMES = 17
SPEED = 18
LAST_FRAME_INDEX = 19
NUMBER_OF_PARAMS = 20

# JPEG Specific Parameters
QUALITY_FACTOR = NUMBER_OF_PARAMS + 0

# MPEG Specific Parameters
END_OF_SEQUENCE = NUMBER_OF_PARAMS + 0

# RTR Specific Parameters
QUALITY_LEVEL = NUMBER_OF_PARAMS + 0
ZOOM_X = NUMBER_OF_PARAMS + 1
ZOOM_Y = NUMBER_OF_PARAMS + 2

#
# Parameter value types
#
ENUM_VALUE = 0                          # only certain constant values are valid
RANGE_VALUE = 1                         # any value in a given range is valid
FLOATING_ENUM_VALUE = 2                 # only certain constant floating point values are valid
FLOATING_RANGE_VALUE = 3                # any value in a given floating point range is valid

#
# Algorithm Functionality
#
DECOMPRESSOR = 1
COMPRESSOR = 2
CODEC = 3

#
# Buffer types
#
NONE = 0
FRAME = 1
DATA = 2

#
# Frame types
#
NONE = 0
KEYFRAME = 1
INTRA = 1
PREDICTED = 2
BIDIRECTIONAL = 3

#
# Orientations
#
TOP_DOWN = 0
BOTTOM_UP = 1

#
# SGI Proprietary Algorithm Header Start Code
#
HEADER_START_CODE = 0xc1C0DEC

#
# error codes
#

BAD_NO_BUFFERSPACE =  -2                # no space for internal buffers
BAD_PVBUFFER =  -3                      # param/val buffer doesn't make sense
BAD_BUFFERLENGTH_NEG =  -4              # negative buffer length
BAD_BUFFERLENGTH_ODD =  -5              # odd length parameter/value buffer
BAD_PARAM =  -6                         # invalid parameter
BAD_COMPRESSION_SCHEME =  -7            # compression scheme parameter invalid
BAD_COMPRESSOR_HANDLE =  -8             # compression handle parameter invalid
BAD_COMPRESSOR_HANDLE_POINTER = -9      # compression handle pointer invalid
BAD_BUFFER_HANDLE = -10                 # buffer handle invalid
BAD_BUFFER_QUERY_SIZE = -11             # buffer query size too large
JPEG_ERROR = -12                        # error from libjpeg
BAD_FRAME_SIZE = -13                    # frame size invalid
PARAM_OUT_OF_RANGE = -14                # parameter out of range
ADDED_ALGORITHM_ERROR = -15             # added algorithm had a unique error
BAD_ALGORITHM_TYPE = -16                # bad algorithm type
BAD_ALGORITHM_NAME = -17                # bad algorithm name
BAD_BUFFERING = -18                     # bad buffering calls
BUFFER_NOT_CREATED = -19                # buffer not created
BAD_BUFFER_EXISTS = -20                 # buffer already created
BAD_INTERNAL_FORMAT = -21               # invalid internal format
BAD_BUFFER_POINTER = -22                # invalid buffer pointer
FRAME_BUFFER_SIZE_ZERO = -23            # frame buffer has zero size
BAD_STREAM_HEADER = -24                 # invalid stream header

BAD_LICENSE = -25                       # netls license not valid
AWARE_ERROR = -26                       # error from libawcmp
